@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: common.h 3033 2005-08-21 13:06:50Z mosu $
+   $Id: common.h 3216 2006-04-28 14:52:55Z mosu $
 
    definitions used in all programs, helper functions
 
@@ -55,7 +55,7 @@ namespace libebml {
 
 using namespace libebml;
 
-#define VERSIONNAME "Another White Dash"
+#define VERSIONNAME "What Do You Take Me For"
 #define VERSIONINFO "mkvmerge v" VERSION " ('" VERSIONNAME "')"
 #define BUGMSG _("This should not have happened. Please contact the author " \
                  "Moritz Bunkus <moritz@bunkus.org> with this error/warning " \
@@ -236,9 +236,10 @@ split(const string &src,
 string MTX_DLL_API join(const char *pattern, vector<string> &strings);
 void MTX_DLL_API strip(string &s, bool newlines = false);
 void MTX_DLL_API strip(vector<string> &v, bool newlines = false);
+string &MTX_DLL_API shrink_whitespace(string &s);
 string MTX_DLL_API escape(const string &src);
 string MTX_DLL_API unescape(const string &src);
-string MTX_DLL_API escape_xml(const string &src, bool escape_quotes = false);
+string MTX_DLL_API escape_xml(const string &src);
 string MTX_DLL_API create_xml_node_name(const char *name, const char **atts);
 bool MTX_DLL_API starts_with(const string &s, const char *start,
                              int maxlen = -1);
@@ -281,12 +282,15 @@ bool MTX_DLL_API parse_double(const char *s, double &value);
 int MTX_DLL_API get_arg_len(const char *fmt, ...);
 int MTX_DLL_API get_varg_len(const char *fmt, va_list ap);
 
+int MTX_DLL_API int_log2(uint32_t value);
+
 extern int MTX_DLL_API verbose;
 
 #define foreach(it, vec) for (it = (vec).begin(); it != (vec).end(); it++)
 #define mxfind(value, cont) std::find(cont.begin(), cont.end(), value)
 #define mxfind2(it, value, cont) \
   ((id = std::find((cont).begin(), (cont).end(), value)) != (cont).end())
+#define map_has_key(m, k) ((m).end() != (m).find(k))
 
 class MTX_DLL_API bitvalue_c {
 private:
