@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ogm.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: r_ogm.cpp 3210 2006-04-28 08:48:49Z mosu $
 
    OGG media stream reader
 
@@ -68,9 +68,8 @@ free_string_array(char **comments) {
 }
 
 static char **
-extract_vorbis_comments(unsigned char *buffer,
-                        int size) {
-  mm_mem_io_c in(buffer, size);
+extract_vorbis_comments(const memory_cptr &mem) {
+  mm_mem_io_c in(mem->get(), mem->get_size());
   char **comments;
   uint32_t i, n, len;
 
@@ -122,8 +121,8 @@ fhe_read_cb(const FLAC__StreamDecoder *decoder,
   memcpy(buffer, op.packet, op.bytes);
   *bytes = op.bytes;
   fhe->num_packets++;
-  mxverb(2, FPFX "read packet number %lld with %ld bytes\n", fhe->num_packets,
-         op.bytes);
+  mxverb(2, FPFX "read packet number " LLD " with %ld bytes\n",
+         fhe->num_packets, op.bytes);
 
   return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
 }
@@ -229,7 +228,7 @@ flac_header_extractor_c::extract() {
     return false;
   }
   result = (int)FLAC__stream_decoder_process_until_end_of_stream(decoder);
-  mxverb(2, FPFX "extract, result: %d, mdp: %d, num_header_packets: %lld\n",
+  mxverb(2, FPFX "extract, result: %d, mdp: %d, num_header_packets: " LLD "\n",
          result, metadata_parsed, num_header_packets);
 
   return metadata_parsed;
@@ -397,6 +396,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
   generic_packetizer_c *ptzr;
   double fps;
   int width, height;
+  memory_cptr codecprivate;
 
   if ((tid < 0) || (tid >= sdemuxers.size()))
     return;
@@ -404,7 +404,7 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
   if ((dmx->ptzr == -1) && dmx->in_use) {
     memset(&bih, 0, sizeof(alBITMAPINFOHEADER));
-    sth = (stream_header *)&dmx->packet_data[0][1];
+    sth = (stream_header *)&dmx->packet_data[0]->get()[1];
     ti.private_data = NULL;
     ti.private_size = 0;
     ti.id = tid;
@@ -476,9 +476,11 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
       case OGM_STREAM_TYPE_AAC:
         aac_info_extracted = false;
-        if (dmx->packet_sizes[0] >= (sizeof(stream_header) + 5)) {
-          if (parse_aac_data(&dmx->packet_data[0][sizeof(stream_header) + 5],
-                             dmx->packet_sizes[0] - sizeof(stream_header) - 5,
+        if (dmx->packet_data[0]->get_size() >= (sizeof(stream_header) + 5)) {
+          if (parse_aac_data(dmx->packet_data[0]->get() +
+                             sizeof(stream_header) + 5,
+                             dmx->packet_data[0]->get_size() -
+                             sizeof(stream_header) - 5,
                              profile, channels, sample_rate,
                              output_sample_rate, sbr)) {
             aac_info_extracted = true;
@@ -492,8 +494,8 @@ ogm_reader_c::create_packetizer(int64_t tid) {
           sample_rate = get_uint64_le(&sth->samples_per_unit);
           profile = AAC_PROFILE_LC;
         }
-        mxverb(2, "ogm_reader: %lld/%s: profile %d, channels %d, sample_rate "
-               "%d, sbr %d, output_sample_rate %d, ex %d\n", ti.id,
+        mxverb(2, "ogm_reader: " LLD "/%s: profile %d, channels %d, "
+               "sample_rate %d, sbr %d, output_sample_rate %d, ex %d\n", ti.id,
                ti.fname.c_str(),
                profile, channels, sample_rate, (int)sbr, output_sample_rate,
                (int)aac_info_extracted);
@@ -511,8 +513,8 @@ ogm_reader_c::create_packetizer(int64_t tid) {
         vorbis_info_init(&vi);
         vorbis_comment_init(&vc);
         memset(&op, 0, sizeof(ogg_packet));
-        op.packet = dmx->packet_data[0];
-        op.bytes = dmx->packet_sizes[0];
+        op.packet = dmx->packet_data[0]->get();
+        op.bytes = dmx->packet_data[0]->get_size();
         op.b_o_s = 1;
         vorbis_synthesis_headerin(&vi, &vc, &op);
         dmx->vorbis_rate = vi.rate;
@@ -520,9 +522,12 @@ ogm_reader_c::create_packetizer(int64_t tid) {
         vorbis_comment_clear(&vc);
         ptzr =
           new vorbis_packetizer_c(this,
-                                  dmx->packet_data[0], dmx->packet_sizes[0],
-                                  dmx->packet_data[1], dmx->packet_sizes[1],
-                                  dmx->packet_data[2], dmx->packet_sizes[2],
+                                  dmx->packet_data[0]->get(),
+                                  dmx->packet_data[0]->get_size(),
+                                  dmx->packet_data[1]->get(),
+                                  dmx->packet_data[1]->get_size(),
+                                  dmx->packet_data[2]->get(),
+                                  dmx->packet_data[2]->get_size(),
                                   ti);
 
         mxinfo(FMT_TID "Using the Vorbis output module.\n", ti.fname.c_str(),
@@ -544,13 +549,14 @@ ogm_reader_c::create_packetizer(int64_t tid) {
         int size, i;
 
         size = 0;
-        for (i = 1; i < (int)dmx->packet_sizes.size(); i++)
-          size += dmx->packet_sizes[i];
+        for (i = 1; i < (int)dmx->packet_data.size(); i++)
+          size += dmx->packet_data[i]->get_size();
         buf = (unsigned char *)safemalloc(size);
         size = 0;
-        for (i = 1; i < (int)dmx->packet_sizes.size(); i++) {
-          memcpy(&buf[size], dmx->packet_data[i], dmx->packet_sizes[i]);
-          size += dmx->packet_sizes[i];
+        for (i = 1; i < (int)dmx->packet_data.size(); i++) {
+          memcpy(&buf[size], dmx->packet_data[i]->get(),
+                 dmx->packet_data[i]->get_size());
+          size += dmx->packet_data[i]->get_size();
         }
         ptzr = new flac_packetizer_c(this, buf, size, ti);
         safefree(buf);
@@ -560,6 +566,21 @@ ogm_reader_c::create_packetizer(int64_t tid) {
 
         break;
 #endif
+      case OGM_STREAM_TYPE_THEORA:
+        codecprivate = lace_memory_xiph(dmx->packet_data);
+        ti.private_data = codecprivate->get();
+        ti.private_size = codecprivate->get_size();
+
+        fps = (double)dmx->theora.frn / (double)dmx->theora.frd;
+        ptzr = new video_packetizer_c(this, MKV_V_THEORA, fps, dmx->theora.fmbw,
+                                      dmx->theora.fmbh, ti);
+        mxinfo(FMT_TID "Using the Theora video output module.\n", ti.fname.c_str(),
+               (int64_t)tid);
+
+        ti.private_data = NULL;
+
+        break;
+
       default:
         die("ogm_reader: Don't know how to create a packetizer for stream "
             "type %d.\n", (int)dmx->stype);
@@ -592,7 +613,8 @@ ogm_reader_c::packet_available() {
     return 0;
 
   for (i = 0; i < sdemuxers.size(); i++)
-    if (!PTZR(sdemuxers[i]->ptzr)->packet_available())
+    if ((-1 != sdemuxers[i]->ptzr) &&
+        !PTZR(sdemuxers[i]->ptzr)->packet_available())
       return 0;
 
   return 1;
@@ -635,8 +657,10 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
   ogg_stream_pagein(&dmx->os, og);
   ogg_stream_packetout(&dmx->os, &op);
 
-  dmx->packet_data.push_back((unsigned char *)safememdup(op.packet, op.bytes));
-  dmx->packet_sizes.push_back(op.bytes);
+  dmx->packet_data.push_back(memory_cptr(new memory_c((unsigned char *)
+                                                      safememdup(op.packet,
+                                                                 op.bytes),
+                                                      op.bytes, true)));
   dmx->serialno = ogg_page_serialno(og);
 
   /*
@@ -648,11 +672,31 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
       return;
 
     dmx->stype = OGM_STREAM_TYPE_VORBIS;
+    dmx->num_header_packets = 3;
     dmx->in_use = true;
 
     return;
   }
 
+  if ((op.bytes >= 7) && !strncmp((char *)&op.packet[1], "theora", 6)) {
+    if (!demuxing_requested('v', sdemuxers.size() - 1))
+      return;
+
+    dmx->stype = OGM_STREAM_TYPE_THEORA;
+    dmx->num_header_packets = 3;
+    dmx->in_use = true;
+
+    try {
+      theora_parse_identification_header(op.packet, op.bytes, dmx->theora);
+    } catch (error_c &e) {
+      mxerror(FMT_TID "The Theora identifaction header could not be parsed "
+              "(%s).\n", ti.fname.c_str(), (int64_t)sdemuxers.size() - 1,
+              e.get_error().c_str());
+    }
+
+    return;
+  }
+
   // FLAC
   if ((op.bytes >= 4) && !strncmp((char *)op.packet, "fLaC", 4)) {
     if (!demuxing_requested('a', sdemuxers.size() - 1))
@@ -690,7 +734,6 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
         return;
 
       dmx->stype = OGM_STREAM_TYPE_VIDEO;
-      dmx->native_mode = false;
       if (video_fps < 0)
         video_fps = 10000000.0 / (float)get_uint64_le(&sth->time_unit);
       dmx->in_use = true;
@@ -722,7 +765,6 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
         return;
       }
 
-      dmx->native_mode = false;
       dmx->in_use = true;
 
       return;
@@ -733,7 +775,6 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
         return;
 
       dmx->stype = OGM_STREAM_TYPE_TEXT;
-      dmx->native_mode = false;
       dmx->in_use = true;
 
       return;
@@ -760,14 +801,26 @@ ogm_reader_c::process_page(ogg_page *og) {
   ogg_packet op;
   int duration_len, eos, i;
   long duration;
+  bool last_granulepos_set;
 
   duration = 0;
   dmx = find_demuxer(ogg_page_serialno(og));
-  if (dmx == NULL)
+  if ((NULL == dmx) || (-1 == dmx->ptzr))
     return;
 
+  if ((-1 != ogg_page_granulepos(og)) &&
+      (ogg_page_granulepos(og) < dmx->last_granulepos)) {
+    mxwarn(FMT_TID "The timecodes for this stream have been reset in the "
+           "middle of the file. This is not supported. The current packet "
+           "will be discarded.\n", ti.fname.c_str(),
+           (int64_t)dmx->serialno);
+    return;
+  }
+
   debug_enter("ogm_reader_c::process_page");
 
+  last_granulepos_set = false;
+
   ogg_stream_pagein(&dmx->os, og);
   while (ogg_stream_packetout(&dmx->os, &op) == 1) {
     eos = op.e_o_s;
@@ -778,8 +831,7 @@ ogm_reader_c::process_page(ogg_page *og) {
       if (dmx->units_processed <= dmx->flac_header_packets)
         continue;
       for (i = 0; i < (int)dmx->nh_packet_data.size(); i++) {
-        memory_c *mem = new memory_c(dmx->nh_packet_data[i],
-                                     dmx->nh_packet_sizes[i], true);
+        memory_c *mem = dmx->nh_packet_data[i]->clone();
         PTZR(dmx->ptzr)->process(new packet_t(mem, 0));
       }
       dmx->nh_packet_data.clear();
@@ -800,10 +852,44 @@ ogm_reader_c::process_page(ogg_page *og) {
         return;
       }
 
+      last_granulepos_set = true;
+
       continue;
     }
 #endif
 
+    if (dmx->stype == OGM_STREAM_TYPE_THEORA) {
+      int keyframe_number, non_keyframe_number;
+      int64_t timecode, duration, bref;
+
+      if ((0 == op.bytes) || (0 != (op.packet[0] & 0x80)))
+        continue;
+
+      keyframe_number = ogg_page_granulepos(og) >> dmx->theora.kfgshift;
+      non_keyframe_number = ogg_page_granulepos(og) & dmx->theora.kfgshift;
+
+      timecode = (int64_t)(1000000000.0 * dmx->units_processed *
+                           dmx->theora.frd / dmx->theora.frn);
+      duration = (int64_t)(1000000000.0 * dmx->theora.frd / dmx->theora.frn);
+      bref = (keyframe_number != dmx->last_keyframe_number) &&
+        (0 == non_keyframe_number) ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC;
+      PTZR(dmx->ptzr)->process(new packet_t(new memory_c(op.packet, op.bytes,
+                                                         false), timecode,
+                                            duration, bref, VFT_NOBFRAME));
+
+      dmx->units_processed++;
+
+      dmx->last_keyframe_number = keyframe_number;
+
+      if (eos) {
+        dmx->eos = 1;
+        debug_leave("ogm_reader_c::process_page");
+        return;
+      }
+
+      continue;
+    }
+
     duration_len = (*op.packet & PACKET_LEN_BITS01) >> 6;
     duration_len |= (*op.packet & PACKET_LEN_BITS2) << 1;
     if ((duration_len > 0) && (op.bytes >= (duration_len + 1)))
@@ -856,6 +942,9 @@ ogm_reader_c::process_page(ogg_page *og) {
     }
   }
 
+  if (!last_granulepos_set)
+    dmx->last_granulepos = ogg_page_granulepos(og);
+
   debug_leave("ogm_reader_c::process_page");
 }
 
@@ -886,16 +975,18 @@ ogm_reader_c::process_header_packets(ogm_demuxer_t *dmx) {
 #if defined HAVE_FLAC_FORMAT_H
     while ((dmx->packet_data.size() < dmx->flac_header_packets) &&
            (ogg_stream_packetout(&dmx->os, &op) == 1)) {
-      dmx->packet_data.push_back((unsigned char *)
-                                 safememdup(op.packet, op.bytes));
-      dmx->packet_sizes.push_back(op.bytes);
+      memory_c *mem = new memory_c((unsigned char *)
+                                   safememdup(op.packet, op.bytes),
+                                   op.bytes, true);
+      dmx->packet_data.push_back(memory_cptr(mem));
     }
     if (dmx->packet_data.size() >= dmx->flac_header_packets)
       dmx->headers_read = true;
     while (ogg_stream_packetout(&dmx->os, &op) == 1) {
-      dmx->nh_packet_data.push_back((unsigned char *)
-                                    safememdup(op.packet, op.bytes));
-      dmx->nh_packet_sizes.push_back(op.bytes);
+      memory_c *mem = new memory_c((unsigned char *)
+                                   safememdup(op.packet, op.bytes),
+                                   op.bytes, true);
+      dmx->nh_packet_data.push_back(memory_cptr(mem));
     }
 #else
     dmx->headers_read = true;
@@ -904,8 +995,14 @@ ogm_reader_c::process_header_packets(ogm_demuxer_t *dmx) {
   }
 
   while (ogg_stream_packetout(&dmx->os, &op) == 1) {
-    if (((dmx->stype != OGM_STREAM_TYPE_VORBIS) || !dmx->native_mode) &&
-        ((op.packet[0] & 1) != 1)) {
+    bool is_header_packet;
+
+    if (OGM_STREAM_TYPE_THEORA == dmx->stype)
+      is_header_packet = ((0x80 <= op.packet[0]) && (0x82 >= op.packet[0]));
+    else
+      is_header_packet = op.packet[0] & 1;
+
+    if (!is_header_packet) {
       mxwarn("ogm_reader: Missing header/comment packets for stream %d in "
              "'%s'. This file is broken but should be muxed correctly. If "
              "not please contact the author Moritz Bunkus "
@@ -914,15 +1011,13 @@ ogm_reader_c::process_header_packets(ogm_demuxer_t *dmx) {
       ogg_stream_reset(&dmx->os);
       return;
     }
-    dmx->packet_data.push_back((unsigned char *)
-                               safememdup(op.packet, op.bytes));
-    dmx->packet_sizes.push_back(op.bytes);
+    memory_c *mem = new memory_c((unsigned char *)
+                                 safememdup(op.packet, op.bytes),
+                                 op.bytes, true);
+    dmx->packet_data.push_back(memory_cptr(mem));
   }
 
-  if (dmx->stype == OGM_STREAM_TYPE_VORBIS) {
-    if (dmx->packet_data.size() == 3)
-      dmx->headers_read = true;
-  } else if (dmx->packet_data.size() == 2)
+  if (dmx->packet_data.size() == dmx->num_header_packets)
     dmx->headers_read = true;
 }
 
@@ -1030,7 +1125,7 @@ ogm_reader_c::identify() {
   mxinfo("File '%s': container: Ogg/OGM%s\n", ti.fname.c_str(), info.c_str());
   for (i = 0; i < sdemuxers.size(); i++) {
     if (sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO) {
-      sth = (stream_header *)&sdemuxers[i]->packet_data[0][1];
+      sth = (stream_header *)(sdemuxers[i]->packet_data[0]->get() + 1);
       memcpy(fourcc, sth->subtype, 4);
       fourcc[4] = 0;
     }
@@ -1053,7 +1148,8 @@ ogm_reader_c::identify() {
             sdemuxers[i]->stype == OGM_STREAM_TYPE_AC3 ||
             sdemuxers[i]->stype == OGM_STREAM_TYPE_AAC ||
             sdemuxers[i]->stype == OGM_STREAM_TYPE_FLAC) ? "audio" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ? "video" :
+           (sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_THEORA) ? "video" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_TEXT ? "subtitles" :
            "unknown",
            sdemuxers[i]->stype == OGM_STREAM_TYPE_VORBIS ? "Vorbis" :
@@ -1062,6 +1158,7 @@ ogm_reader_c::identify() {
            sdemuxers[i]->stype == OGM_STREAM_TYPE_AC3 ? "AC3" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_AAC ? "AAC" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ? fourcc :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_THEORA ? "Theora" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_TEXT ? "text" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_FLAC ? "FLAC" :
            "unknown",
@@ -1074,7 +1171,6 @@ ogm_reader_c::handle_stream_comments() {
   int i, j, cch;
   ogm_demuxer_t *dmx;
   char **comments;
-  const char *iso639_2;
   vector<string> comment, chapters;
   mm_mem_io_c *out;
   string title;
@@ -1091,8 +1187,7 @@ ogm_reader_c::handle_stream_comments() {
     if ((dmx->stype == OGM_STREAM_TYPE_FLAC) ||
         (dmx->packet_data.size() < 2))
       continue;
-    comments = extract_vorbis_comments(dmx->packet_data[1],
-                                       dmx->packet_sizes[1]);
+    comments = extract_vorbis_comments(dmx->packet_data[1]);
     if (comments == NULL)
       continue;
     chapters.clear();
@@ -1105,12 +1200,12 @@ ogm_reader_c::handle_stream_comments() {
         continue;
 
       if (comment[0] == "LANGUAGE") {
-        iso639_2 = map_english_name_to_iso639_2(comment[1].c_str());
-        if (iso639_2 == NULL)
-          iso639_2 = map_iso639_1_to_iso639_2(comment[1].c_str());
-        if ((iso639_2 == NULL) && (is_valid_iso639_2_code(comment[1].c_str())))
-          iso639_2 = comment[1].c_str();
-        if (iso639_2 == NULL) {
+        int index;
+
+        index = map_to_iso639_2_code(comment[1].c_str());
+        if (-1 != index)
+          dmx->language = iso639_languages[index].iso639_2_code;
+        else {
           string lang;
           int pos1, pos2;
 
@@ -1131,14 +1226,10 @@ ogm_reader_c::handle_stream_comments() {
             lang.erase(pos1, pos2 - pos1 + 1);
             pos1 = lang.find("(");
           }
-          for (pos1 = 0; iso639_languages[pos1].iso639_2_code != NULL; pos1++)
-            if (starts_with_case(lang, iso639_languages[pos1].english_name)) {
-              iso639_2 = iso639_languages[pos1].iso639_2_code;
-              break;
-            }
+          index = map_to_iso639_2_code(lang.c_str());
+          if (-1 != index)
+            dmx->language = iso639_languages[index].iso639_2_code;
         }
-        if (iso639_2 != NULL)
-          dmx->language = iso639_2;
 
       } else if (comment[0] == "TITLE")
         title = comment[1];
