@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_input.cpp 2995 2005-07-19 20:01:07Z mosu $
+   $Id: tab_input.cpp 3171 2005-12-17 09:57:40Z mosu $
 
    "input" tab
 
@@ -41,6 +41,53 @@ wxArrayString sorted_iso_codes;
 wxArrayString sorted_charsets;
 bool title_was_present = false;
 
+static const wxChar *predefined_aspect_ratios[] = {
+  wxT(""),
+  wxT("4/3"),
+  wxT("1.66"),
+  wxT("16/9"),
+  wxT("1.85"),
+  wxT("2.00"),
+  wxT("2.21"),
+  wxT("2.35"),
+  NULL
+};
+
+static const wxChar *predefined_fourccs[] = {
+  wxT(""),
+  wxT("DIVX"),
+  wxT("DIV3"),
+  wxT("DX50"),
+  wxT("MP4V"),
+  wxT("XVID"),
+  NULL
+};
+
+void
+set_combobox_selection(wxComboBox *cb,
+                       const wxChar * const string_list[],
+                       const wxString wanted) {
+  int i;
+  bool found;
+
+  cb->SetValue(wanted);
+  found = false;
+  for (i = 0; NULL != string_list[i]; ++i)
+    if (wanted == string_list[i]) {
+      cb->SetSelection(i);
+      found = true;
+      break;
+    }
+}
+
+wxString
+get_combobox_selection(wxComboBox *cb) {
+  if (wxNOT_FOUND != cb->GetSelection())
+    return cb->GetStringSelection();
+  else
+    return cb->GetValue();
+}
+
 class input_drop_target_c: public wxFileDropTarget {
 private:
   tab_input *owner;
@@ -310,12 +357,10 @@ tab_input::tab_input(wxWindow *parent):
   cob_fourcc =
     new wxComboBox(this, ID_CB_FOURCC, wxT(""), wxDefaultPosition,
                    wxDefaultSize, 0, NULL, wxCB_DROPDOWN);
-  cob_fourcc->Append(wxT(""));
-  cob_fourcc->Append(wxT("DIVX"));
-  cob_fourcc->Append(wxT("DIV3"));
-  cob_fourcc->Append(wxT("DX50"));
-  cob_fourcc->Append(wxT("MP4V"));
-  cob_fourcc->Append(wxT("XVID"));
+
+  for (i = 0; NULL != predefined_fourccs[i]; ++i)
+    cob_fourcc->Append(predefined_fourccs[i]);
+
   cob_fourcc->SetToolTip(TIP("Forces the FourCC of the video track to this "
                              "value. Note that this only works for video "
                              "tracks that use the AVI compatibility mode "
@@ -358,14 +403,10 @@ tab_input::tab_input(wxWindow *parent):
   cob_aspect_ratio =
     new wxComboBox(this, ID_CB_ASPECTRATIO, wxT(""), wxDefaultPosition,
                    wxDefaultSize, 0, NULL, wxCB_DROPDOWN);
-  cob_aspect_ratio->Append(wxT(""));
-  cob_aspect_ratio->Append(wxT("4/3"));
-  cob_aspect_ratio->Append(wxT("1.66"));
-  cob_aspect_ratio->Append(wxT("16/9"));
-  cob_aspect_ratio->Append(wxT("1.85"));
-  cob_aspect_ratio->Append(wxT("2.00"));
-  cob_aspect_ratio->Append(wxT("2.21"));
-  cob_aspect_ratio->Append(wxT("2.35"));
+
+  for (i = 0; NULL != predefined_aspect_ratios[i]; ++i)
+    cob_aspect_ratio->Append(predefined_aspect_ratios[i]);
+
   cob_aspect_ratio->SetToolTip(TIP("Sets the display aspect ratio of the "
                                    "track. The format can be either 'a/b' in "
                                    "which case both numbers must be integer "
@@ -634,11 +675,11 @@ tab_input::enable_ar_controls(mmg_track_t *track) {
   bool ar_enabled;
 
   ar_enabled = !track->display_dimensions_selected;
-  cob_aspect_ratio->Enable(ar_enabled);
-  tc_display_width->Enable(!ar_enabled);
-  tc_display_height->Enable(!ar_enabled);
-  rb_aspect_ratio->SetValue(ar_enabled);
-  rb_display_dimensions->SetValue(!ar_enabled);
+  cob_aspect_ratio->Enable(ar_enabled && !track->appending);
+  tc_display_width->Enable(!ar_enabled && !track->appending);
+  tc_display_height->Enable(!ar_enabled && !track->appending);
+  rb_aspect_ratio->SetValue(ar_enabled && !track->appending);
+  rb_display_dimensions->SetValue(!ar_enabled && !track->appending);
 }
 
 void
@@ -859,8 +900,10 @@ tab_input::add_file(const wxString &file_name,
             dims = split(pair[1], wxU("x"));
             if ((dims.size() == 2) && parse_int(wxMB(dims[0]), width) &&
                 parse_int(wxMB(dims[1]), height)) {
-              track->dwidth.Printf(wxT(LLD), width);
-              track->dheight.Printf(wxT(LLD), height);
+              string format;
+              fix_format(LLD, format);
+              track->dwidth.Printf(wxU(format.c_str()), width);
+              track->dheight.Printf(wxU(format.c_str()), height);
               track->display_dimensions_selected = true;
             }
 
@@ -992,7 +1035,7 @@ tab_input::add_file(const wxString &file_name,
     t->enabled = true;
     t->source = new_file_pos;
 
-    fix_format("%s%s (ID %lld, type: %s) from %s", format);
+    fix_format("%s%s (ID " LLD ", type: %s) from %s", format);
     label.Printf(wxU(format.c_str()), t->appending ? wxT("++> ") : wxT(""),
                  t->ctype.c_str(), t->id,
                  t->type == 'a' ? wxT("audio") :
@@ -1046,7 +1089,6 @@ tab_input::on_remove_file(wxCommandEvent &evt) {
   mmg_file_t *f;
   mmg_track_t *t;
   vector<mmg_file_t>::iterator eit;
-  vector<mmg_track_t *>::iterator tit;
   uint32_t i;
 
   if (selected_file == -1)
@@ -1074,17 +1116,15 @@ tab_input::on_remove_file(wxCommandEvent &evt) {
   dont_copy_values_now = true;
 
   i = 0;
-  tit = tracks.begin();
   while (i < tracks.size()) {
     t = tracks[i];
     if (t->source == selected_file) {
       clb_tracks->Delete(i);
-      tracks.erase(tit);
+      tracks.erase(tracks.begin() + i);
     } else {
       if (t->source > selected_file)
         t->source--;
       i++;
-      tit++;
     }
   }
 
@@ -1110,6 +1150,7 @@ tab_input::on_remove_file(wxCommandEvent &evt) {
     st_tracks->Enable(false);
     clb_tracks->Enable(false);
     selected_track = -1;
+    mdlg->remove_output_filename();
   }
 
   dont_copy_values_now = false;
@@ -1185,12 +1226,14 @@ tab_input::on_file_selected(wxCommandEvent &evt) {
   mmg_file_t *f;
   wxString label;
 
+  selected_file = -1;
+  new_sel = lb_input_files->GetSelection();
+  if (0 > new_sel)
+    return;
   b_remove_file->Enable(true);
   b_append_file->Enable(true);
-  selected_file = -1;
   old_track = selected_track;
   selected_track = -1;
-  new_sel = lb_input_files->GetSelection();
   f = &files[new_sel];
   if (f->container == FILE_TYPE_MATROSKA) {
     st_file_options->Enable(true);
@@ -1275,13 +1318,14 @@ tab_input::on_track_selected(wxCommandEvent &evt) {
   tc_tags->SetValue(t->tags);
   cb_default->SetValue(t->default_track);
   cb_aac_is_sbr->SetValue(t->aac_is_sbr);
-  cob_aspect_ratio->SetValue(t->aspect_ratio);
+  set_combobox_selection(cob_aspect_ratio, predefined_aspect_ratios,
+                         t->aspect_ratio);
   tc_display_width->SetValue(t->dwidth);
   tc_display_height->SetValue(t->dheight);
   selected_track = new_sel;
   cob_compression->SetValue(t->compression);
   tc_timecodes->SetValue(t->timecodes);
-  cob_fourcc->SetValue(t->fourcc);
+  set_combobox_selection(cob_fourcc, predefined_fourccs, t->fourcc);
   tc_track_name->SetFocus();
 
   dont_copy_values_now = false;
@@ -1489,8 +1533,8 @@ tab_input::on_value_copy_timer(wxTimerEvent &evt) {
     return;
 
   t = tracks[selected_track];
-  t->aspect_ratio = cob_aspect_ratio->GetValue();
-  t->fourcc = cob_fourcc->GetValue();
+  t->aspect_ratio = get_combobox_selection(cob_aspect_ratio);
+  t->fourcc = get_combobox_selection(cob_fourcc);
 }
 
 void
@@ -1521,7 +1565,7 @@ tab_input::save(wxConfigBase *cfg) {
       cfg->SetPath(s);
       s.Printf(wxT("%c"), t->type);
       cfg->Write(wxT("type"), s);
-      fix_format("%lld", format);
+      fix_format("" LLD, format);
       s.Printf(wxU(format.c_str()), t->id);
       cfg->Write(wxT("id"), s);
       cfg->Write(wxT("enabled"), t->enabled);
@@ -1582,7 +1626,7 @@ tab_input::load(wxConfigBase *cfg,
     return;
   }
 
-  fix_format("%u:%lld", format);
+  fix_format("%u:" LLD, format);
   for (fidx = 0; fidx < num_files; fidx++) {
     mmg_file_t fi;
 
@@ -1703,7 +1747,7 @@ tab_input::load(wxConfigBase *cfg,
       mmg_track_ptr &t = files[fidx].tracks[tidx];
       tracks.push_back(t.get());
 
-      fix_format("%s%s (ID %lld, type: %s) from %s", format);
+      fix_format("%s%s (ID " LLD ", type: %s) from %s", format);
       name = files[fidx].file_name.AfterLast(wxT(PSEP));
       name += wxT(" (") + files[fidx].file_name.BeforeLast(wxT(PSEP)) +
         wxT(")");
@@ -1758,7 +1802,7 @@ tab_input::validate_settings() {
         continue;
 
       tracks_selected = true;
-      fix_format("%lld", format);
+      fix_format("" LLD, format);
       sid.Printf(wxU(format.c_str()), t->id);
 
       s = wxMB(t->delay);
