@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tab_settings.h 2832 2005-02-27 20:39:02Z mosu $
+   $Id: tab_settings.h 2965 2005-06-24 14:58:13Z mosu $
 
    declarations for the settings tab
 
@@ -52,7 +52,7 @@ public:
   void save_preferences();
 
   void save(wxConfigBase *cfg);
-  void load(wxConfigBase *cfg);
+  void load(wxConfigBase *cfg, int version);
   bool validate_settings();
 
   void query_mkvmerge_capabilities();
