@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_video.cpp 3032 2005-08-21 13:05:24Z mosu $
+   $Id: p_video.cpp 3194 2006-02-27 17:06:55Z mosu $
 
    video output module
 
@@ -61,9 +61,9 @@ video_packetizer_c::check_fourcc() {
   if ((hcodec_id == MKV_V_MSCOMP) &&
       (ti.private_data != NULL) &&
       (ti.private_size >= sizeof(alBITMAPINFOHEADER)) &&
-      (ti.fourcc[0] != 0)) {
+      !ti.fourcc.empty()) {
     memcpy(&((alBITMAPINFOHEADER *)ti.private_data)->bi_compression,
-           ti.fourcc, 4);
+           ti.fourcc.c_str(), 4);
     set_codec_private(ti.private_data, ti.private_size);
   }
 
@@ -94,6 +94,12 @@ video_packetizer_c::check_fourcc() {
                 "into a MP4 file with e.g. 'MP4Box' (use Google). Then "
                 "you can use mkvmerge and put the video into a Matroska "
                 "file.\n"
+                "If you're trying to get AVC/h.264 from an AVI into Matroska "
+                "then the commands you need to use are for example:\n"
+                " avi2raw avc.avi raw.264\n"
+                " mp4box -fps 23.976 -add raw.264 avc.mp4\n\n"
+                "Keep in mind to replace '-fps 23.976' with the actual "
+                "frame rate of your source file.\n"
                 "If you really know what you are doing then you can force "
                 "mkvmerge to put this AVC/h.264 track into a Matroska file "
                 "even in VfW mode if you add '--engage allow_avc_in_vfw_mode' "
@@ -127,13 +133,21 @@ video_packetizer_c::set_headers() {
 //           not relative!)
 int
 video_packetizer_c::process(packet_cptr packet) {
+  if ((0.0 == fps) && (-1 == packet->timecode))
+    mxerror(FMT_TID "video_packetizer: The FPS is 0.0 but the reader did "
+            "not provide a timecode for a packet. %s\n",
+            ti.fname.c_str(), (int64_t)ti.id, BUGMSG);
+
   if (packet->timecode == -1)
     packet->timecode = (int64_t)(1000000000.0 * frames_output / fps) +
       duration_shift;
 
-  if (packet->duration == -1)
-    packet->duration = (int64_t)(1000000000.0 / fps);
-  else
+  if (packet->duration == -1) {
+    if (0.0 == fps)
+      packet->duration = 0;
+    else
+      packet->duration = (int64_t)(1000000000.0 / fps);
+  } else if (0.0 != packet->duration)
     duration_shift += packet->duration - (int64_t)(1000000000.0 / fps);
   frames_output++;
 
@@ -157,7 +171,7 @@ video_packetizer_c::process(packet_cptr packet) {
 void
 video_packetizer_c::dump_debug_info() {
   mxdebug("video_packetizer_c: queue: %u; frames_output: %d; "
-          "ref_timecode: %lld\n", (unsigned int)packet_queue.size(),
+          "ref_timecode: " LLD "\n", (unsigned int)packet_queue.size(),
           frames_output, ref_timecode);
 }
 
@@ -179,7 +193,7 @@ video_packetizer_c::can_connect_to(generic_packetizer_c *src,
     error_message = mxsprintf("The codec's private data does not match "
                               "(lengths: %d and %d).", ti.private_size,
                               vsrc->ti.private_size);
-    return CAN_CONNECT_NO_PARAMETERS;
+    return CAN_CONNECT_MAYBE_CODECPRIVATE;
   }
   return CAN_CONNECT_YES;
 }
@@ -202,6 +216,7 @@ mpeg1_2_video_packetizer_c(generic_reader_c *_reader,
   set_codec_id(mxsprintf("V_MPEG%d", _version));
   if (!ti.aspect_ratio_given && !ti.display_dimensions_given) {
     if ((_dwidth > 0) && (_dheight > 0)) {
+      aspect_ratio_extracted = true;
       ti.display_dimensions_given = true;
       ti.display_width = _dwidth;
       ti.display_height = _dheight;
@@ -218,13 +233,13 @@ mpeg1_2_video_packetizer_c::process(packet_cptr packet) {
   int new_bytes, state;
 
   if (fps < 0.0)
-    extract_fps(packet->memory->data, packet->memory->size);
+    extract_fps(packet->data->get(), packet->data->get_size());
 
   if (!aspect_ratio_extracted)
-    extract_aspect_ratio(packet->memory->data, packet->memory->size);
+    extract_aspect_ratio(packet->data->get(), packet->data->get_size());
 
   if (framed) {
-    if (0 != packet->memory->size)
+    if (0 != packet->data->get_size())
       return video_packetizer_c::process(packet);
     else
       return FILE_STATUS_MOREDATA;
@@ -235,9 +250,9 @@ mpeg1_2_video_packetizer_c::process(packet_cptr packet) {
       (state == MPV_PARSER_STATE_ERROR))
     return FILE_STATUS_DONE;
 
-  memory_cptr old_memory = packet->memory;
-  data_ptr = old_memory->data;
-  new_bytes = old_memory->size;
+  memory_cptr old_memory = packet->data;
+  data_ptr = old_memory->get();
+  new_bytes = old_memory->get_size();
 
   do {
     int bytes_to_add;
@@ -343,13 +358,13 @@ mpeg4_p2_video_packetizer_c(generic_reader_c *_reader,
                             bool _input_is_native,
                             track_info_c &_ti):
   video_packetizer_c(_reader, MKV_V_MPEG4_ASP, _fps, _width, _height, _ti),
-  timecodes_generated(0), last_i_p_frame(0), previous_timecode(0),
+  timecodes_generated(0), last_i_p_frame(-1), previous_timecode(0),
   aspect_ratio_extracted(false), input_is_native(_input_is_native),
-  output_is_native(hack_engaged(ENGAGE_NATIVE_MPEG4)) {
+  output_is_native(hack_engaged(ENGAGE_NATIVE_MPEG4)),
+  size_extracted(false) {
 
-  if (input_is_native && !output_is_native)
-    mxerror("mkvmerge does not support muxing from native MPEG-4 to "
-            "AVI compatibility mode at the moment.\n");
+  if (input_is_native)
+    output_is_native = true;
 
   if (!output_is_native) {
     set_codec_id(MKV_V_MSCOMP);
@@ -369,8 +384,10 @@ mpeg4_p2_video_packetizer_c(generic_reader_c *_reader,
 
 int
 mpeg4_p2_video_packetizer_c::process(packet_cptr packet) {
+  if (!size_extracted)
+    extract_size(packet->data->get(), packet->data->get_size());
   if (!aspect_ratio_extracted)
-    extract_aspect_ratio(packet->memory->data, packet->memory->size);
+    extract_aspect_ratio(packet->data->get(), packet->data->get_size());
 
   if (input_is_native == output_is_native)
     return video_packetizer_c::process(packet);
@@ -389,11 +406,12 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
   if (NULL == ti.private_data) {
     memory_c *config_data;
 
-    config_data = mpeg4_p2_parse_config_data(packet->memory->data,
-                                             packet->memory->size);
+    config_data = mpeg4_p2_parse_config_data(packet->data->get(),
+                                             packet->data->get_size());
     if (NULL != config_data) {
-      ti.private_data = config_data->grab();
-      ti.private_size = config_data->size;
+      ti.private_data = (unsigned char *)safememdup(config_data->get(),
+                                                    config_data->get_size());
+      ti.private_size = config_data->get_size();
       delete config_data;
       fix_codec_string();
       set_codec_private(ti.private_data, ti.private_size);
@@ -405,7 +423,7 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
               "native mode.\n");
   }
 
-  mpeg4_p2_find_frame_types(packet->memory->data, packet->memory->size,
+  mpeg4_p2_find_frame_types(packet->data->get(), packet->data->get_size(),
                             frames);
 
   // Add a timecode and a duration if they've been given.
@@ -434,13 +452,8 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
     if (-1 == packet->duration)
       available_durations.push_back((int64_t)(1000000000.0 / fps));
 
-    // Copy the data. If there's only one frame in this packet then
-    // we might save a memcpy.
-    if ((1 == frames.size()) && (frame->size == packet->memory->size))
-      frame->data = packet->memory->grab();
-    else
-      frame->data = (unsigned char *)
-        safememdup(&packet->memory->data[frame->pos], frame->size);
+    frame->data = (unsigned char *)
+      safememdup(packet->data->get() + frame->pos, frame->size);
     queued_frames.push_back(*frame);
   }
 
@@ -526,7 +539,7 @@ mpeg4_p2_video_packetizer_c::flush_frames_maybe(frame_type_e next_frame) {
       more frames than the timecode queue. For example: The last frame
       contained two frames, a P and a B frame. Right afterwards the
       end of the file is reached. In this case a dummy frame is missing.
- 
+
    Both cases can be solved if the source file provides a FPS for this
    track. Otherwise such frames have to be dropped.
 */
@@ -536,12 +549,12 @@ mpeg4_p2_video_packetizer_c::handle_missing_timecodes(bool end_of_file) {
     while (available_timecodes.size() < queued_frames.size()) {
       previous_timecode = (int64_t)(previous_timecode +  1000000000.0 / fps);
       available_timecodes.push_back(previous_timecode);
-      mxverb(3, "mpeg4_p2::flush_frames(): Needed new timecode %lld\n",
+      mxverb(3, "mpeg4_p2::flush_frames(): Needed new timecode " LLD "\n",
              previous_timecode);
     }
     while (available_durations.size() < queued_frames.size()) {
       available_durations.push_back((int64_t)(1000000000.0 / fps));
-      mxverb(3, "mpeg4_p2::flush_frames(): Needed new duration %lld\n",
+      mxverb(3, "mpeg4_p2::flush_frames(): Needed new duration " LLD "\n",
              available_durations.back());
     }
 
@@ -551,12 +564,12 @@ mpeg4_p2_video_packetizer_c::handle_missing_timecodes(bool end_of_file) {
 
     if (available_timecodes.size() < available_durations.size()) {
       num_dropped = queued_frames.size() - available_timecodes.size();
-      available_durations.erase(available_durations.begin() + 
+      available_durations.erase(available_durations.begin() +
                                 available_timecodes.size(),
                                 available_durations.end());
     } else {
       num_dropped = queued_frames.size() - available_durations.size();
-      available_timecodes.erase(available_timecodes.begin() + 
+      available_timecodes.erase(available_timecodes.begin() +
                                 available_durations.size(),
                                 available_timecodes.end());
     }
@@ -585,52 +598,47 @@ mpeg4_p2_video_packetizer_c::handle_missing_timecodes(bool end_of_file) {
 
 void
 mpeg4_p2_video_packetizer_c::flush_frames(bool end_of_file) {
-  int i, num_bframes, b_offset;
-  int64_t b_bref, b_fref;
+  int i;
+
+  if (queued_frames.empty())
+    return;
 
   if ((available_timecodes.size() < queued_frames.size()) ||
       (available_durations.size() < queued_frames.size()))
     handle_missing_timecodes(end_of_file);
 
-  if ((2 <= queued_frames.size()) && (FRAME_TYPE_B != queued_frames[1].type))
-    b_offset = 1;
-  else
-    b_offset = 0;
-
-  num_bframes = 0;
-  b_fref = -1;
-  b_bref = last_i_p_frame;
-  for (i = 0; i < queued_frames.size(); ++i) {
-    if (FRAME_TYPE_I == queued_frames[i].type) {
-      queued_frames[i].timecode = available_timecodes[0];
-      queued_frames[i].duration = available_durations[0];
-      queued_frames[i].bref = -1;
-      queued_frames[i].fref = -1;
-      if (-1 == last_i_p_frame) {
-        last_i_p_frame = queued_frames[i].timecode;
-        b_bref = queued_frames[i].timecode;
-      }
-      b_fref = queued_frames[i].timecode;
-
-    } else if (FRAME_TYPE_P == queued_frames[i].type) {
-      queued_frames[i].timecode =
-        available_timecodes[queued_frames.size() - 1];
-      queued_frames[i].duration =
-        available_durations[queued_frames.size() - 1];
-      queued_frames[i].bref = last_i_p_frame;
-      last_i_p_frame = queued_frames[i].timecode;
-      b_fref = last_i_p_frame;
-      queued_frames[i].fref = -1;
-
-    } else {
-      queued_frames[i].timecode = available_timecodes[num_bframes + b_offset];
-      queued_frames[i].duration = available_durations[num_bframes + b_offset];
-      queued_frames[i].bref = b_bref;
-      queued_frames[i].fref = b_fref;
-      ++num_bframes;
-    }      
+  // Very first frame?
+  if (-1 == last_i_p_frame) {
+    video_frame_t &frame = queued_frames.front();
+    last_i_p_frame = available_timecodes[0];
+    add_packet(new packet_t(new memory_c(frame.data, frame.size, true),
+                            last_i_p_frame, available_durations[0], -1, -1));
+    available_timecodes.erase(available_timecodes.begin(),
+                              available_timecodes.begin() + 1);
+    available_durations.erase(available_durations.begin(),
+                              available_durations.begin() + 1);
+    queued_frames.pop_front();
+    flush_frames(end_of_file);
+    return;
   }
 
+  queued_frames[0].timecode = available_timecodes[queued_frames.size() - 1];
+  queued_frames[0].duration = available_durations[queued_frames.size() - 1];
+  queued_frames[0].fref = -1;
+  if (FRAME_TYPE_I == queued_frames[0].type)
+    queued_frames[0].bref = -1;
+  else
+    queued_frames[0].bref = last_i_p_frame;
+
+  for (i = 1; queued_frames.size() > i; ++i) {
+    queued_frames[i].timecode = available_timecodes[i - 1];
+    queued_frames[i].duration = available_durations[i - 1];
+    queued_frames[i].fref = queued_frames[0].timecode;
+    queued_frames[i].bref = last_i_p_frame;
+  }
+
+  last_i_p_frame = queued_frames[0].timecode;
+
   for (i = 0; i < queued_frames.size(); ++i)
     add_packet(new packet_t(new memory_c(queued_frames[i].data,
                                          queued_frames[i].size, true),
@@ -671,7 +679,7 @@ mpeg4_p2_video_packetizer_c::extract_aspect_ratio(const unsigned char *buffer,
       (float)hvideo_pixel_height * (float)num / (float)den;
     generic_packetizer_c::set_headers();
     rerender_track_headers();
-    mxinfo("Track %lld of '%s': Extracted the aspect ratio information "
+    mxinfo("Track " LLD " of '%s': Extracted the aspect ratio information "
            "from the MPEG4 layer 2 video data and set the display dimensions "
            "to %u/%u.\n", (int64_t)ti.id, ti.fname.c_str(),
            (uint32_t)ti.display_width, (uint32_t)ti.display_height);
@@ -679,6 +687,37 @@ mpeg4_p2_video_packetizer_c::extract_aspect_ratio(const unsigned char *buffer,
     aspect_ratio_extracted = true;
 }
 
+void
+mpeg4_p2_video_packetizer_c::extract_size(const unsigned char *buffer,
+                                          int size) {
+  uint32_t width, height;
+
+  if (mpeg4_p2_extract_size(buffer, size, width, height)) {
+    size_extracted = true;
+    if (!reader->appending &&
+        ((width != hvideo_pixel_width) || (height != hvideo_pixel_height))) {
+      set_video_pixel_width(width);
+      set_video_pixel_height(height);
+      if (!output_is_native &&
+          (ti.private_size >= sizeof(alBITMAPINFOHEADER))) {
+        alBITMAPINFOHEADER *bih = (alBITMAPINFOHEADER *)ti.private_data;
+        put_uint32_le(&bih->bi_width, width);
+        put_uint32_le(&bih->bi_height, height);
+        set_codec_private(ti.private_data, ti.private_size);
+      }
+      generic_packetizer_c::set_headers();
+      rerender_track_headers();
+      mxinfo("Track " LLD " of '%s': The extracted values for video width and "
+             "height from the MPEG4 layer 2 video data bitstream differ from "
+             "what the values in the source container. The ones from the "
+             "video data bitstream (%ux%u) will be used.\n",
+             (int64_t)ti.id, ti.fname.c_str(), width, height);
+    }
+
+  } else if (50 <= frames_output)
+    aspect_ratio_extracted = true;
+}
+
 // ----------------------------------------------------------------
 
 mpeg4_p10_video_packetizer_c::
@@ -696,44 +735,74 @@ mpeg4_p10_video_packetizer_c(generic_reader_c *_reader,
 void
 mpeg4_p10_video_packetizer_c::extract_aspect_ratio() {
   uint32_t num, den;
+  unsigned char *priv;
 
-  if (ti.aspect_ratio_given || ti.display_dimensions_given)
-    return;
-
+  priv = ti.private_data;
   if (mpeg4_p10_extract_par(ti.private_data, ti.private_size, num, den) &&
       (0 != num) && (0 != den)) {
-    double par = (double)num / (double)den;
+    if (!ti.aspect_ratio_given && !ti.display_dimensions_given) {
+      double par = (double)num / (double)den;
 
-    if (par >= 1) {
-      ti.display_width = irnd(width * par);
-      ti.display_height = height;
+      if (par >= 1) {
+        ti.display_width = irnd(width * par);
+        ti.display_height = height;
 
-    } else {
-      ti.display_width = width;
-      ti.display_height = irnd(height / par);
+      } else {
+        ti.display_width = width;
+        ti.display_height = irnd(height / par);
 
+      }
+
+      ti.display_dimensions_given = true;
+      mxinfo("Track " LLD " of '%s': Extracted the aspect ratio information "
+             "from the MPEG-4 layer 10 (AVC) video data and set the display "
+             "dimensions to %u/%u.\n", (int64_t)ti.id, ti.fname.c_str(),
+             (uint32_t)ti.display_width, (uint32_t)ti.display_height);
     }
 
-    ti.display_dimensions_given = true;
-    mxinfo("Track %lld of '%s': Extracted the aspect ratio information "
-           "from the MPEG-4 layer 10 (AVC) video data and set the display "
-           "dimensions to %u/%u.\n", (int64_t)ti.id, ti.fname.c_str(),
-           (uint32_t)ti.display_width, (uint32_t)ti.display_height);
+    set_codec_private(ti.private_data, ti.private_size);
   }
+
+  if (priv != ti.private_data)
+    safefree(priv);
 }
 
 int
 mpeg4_p10_video_packetizer_c::process(packet_cptr packet) {
-  if ((0 > packet->bref) && (0 > packet->fref))
-    ref_timecode = packet->timecode;
-  else {
+  if (VFT_PFRAMEAUTOMATIC == packet->bref) {
     packet->fref = -1;
     packet->bref = ref_timecode;
-    if (packet->timecode > ref_timecode)
-      ref_timecode = packet->timecode;
   }
 
+  ref_timecode = packet->timecode;
+
   add_packet(packet);
 
   return FILE_STATUS_MOREDATA;
 }
+
+connection_result_e
+mpeg4_p10_video_packetizer_c::can_connect_to(generic_packetizer_c *src,
+                                             string &error_message) {
+  connection_result_e result;
+  mpeg4_p10_video_packetizer_c *vsrc;
+
+  vsrc = dynamic_cast<mpeg4_p10_video_packetizer_c *>(src);
+  if (NULL == vsrc)
+    return CAN_CONNECT_NO_FORMAT;
+
+  result = video_packetizer_c::can_connect_to(src, error_message);
+  if (CAN_CONNECT_YES != result)
+    return result;
+
+  if ((NULL != ti.private_data) &&
+      memcmp(ti.private_data, vsrc->ti.private_data, ti.private_size)) {
+    error_message = mxsprintf("The codec's private data does not match."
+                              "Both have the same length (%d) but different "
+                              "content.", ti.private_size);
+    return CAN_CONNECT_MAYBE_CODECPRIVATE;
+  }
+
+  return CAN_CONNECT_YES;
+}
+
